@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
  
@@ -353,7 +355,7 @@ void sql_token_free(sql_token *token);
 /**
  * get the name for a token-id
  */
-const gchar *sql_token_get_name(sql_token_id token_id);
+const gchar *sql_token_get_name(sql_token_id token_id, size_t *name_len);
 
 /**
  * get the token_id for a literal
@@ -363,7 +365,18 @@ const gchar *sql_token_get_name(sql_token_id token_id);
  * @param name     a SQL keyword
  * @return         TK_SQL_(keyword) or TK_LITERAL
  */
-sql_token_id sql_token_get_id(const gchar *name);
+sql_token_id sql_token_get_id(const gchar *name) G_GNUC_DEPRECATED;
+
+/**
+ * get the token_id for a literal
+ *
+ * @internal       only used to drive the test-cases 
+ *
+ * @param name     a SQL keyword
+ * @return         TK_SQL_(keyword) or TK_LITERAL
+ */
+sql_token_id sql_token_get_id_len(const gchar *name, size_t name_len);
+
 
 /**
  * scan a string into SQL tokens
