@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2007, 2009, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
 
@@ -239,7 +241,14 @@ NETWORK_MYSQLD_PLUGIN_PROTO(server_read_auth) {
 
 	auth = network_mysqld_auth_response_new();
 	if (network_mysqld_proto_get_auth_response(&packet, auth)) {
-		g_assert_not_reached();
+		network_mysqld_auth_response_free(auth);
+		return NETWORK_SOCKET_ERROR;
+	}
+	if (!(auth->capabilities & CLIENT_PROTOCOL_41)) {
+		/* should use packet-id 0 */
+		network_mysqld_queue_append(con->client, con->client->send_queue, C("\xff\xd7\x07" "4.0 protocol is not supported"));
+		network_mysqld_auth_response_free(auth);
+		return NETWORK_SOCKET_ERROR;
 	}
 	
 	con->client->response = auth;
@@ -531,8 +540,8 @@ static GOptionEntry * network_mysqld_admin_plugin_get_options(chassis_plugin_con
 	static GOptionEntry config_entries[] = 
 	{
 		{ "admin-address",            0, 0, G_OPTION_ARG_STRING, NULL, "listening address:port of the admin-server (default: :4041)", "<host:port>" },
-		{ "admin-username",           0, 0, G_OPTION_ARG_STRING, NULL, "username to allow to log in (default: root)", "<string>" },
-		{ "admin-password",           0, 0, G_OPTION_ARG_STRING, NULL, "password to allow to log in (default: )", "<string>" },
+		{ "admin-username",           0, 0, G_OPTION_ARG_STRING, NULL, "username to allow to log in", "<string>" },
+		{ "admin-password",           0, 0, G_OPTION_ARG_STRING, NULL, "password to allow to log in", "<string>" },
 		{ "admin-lua-script",         0, 0, G_OPTION_ARG_FILENAME, NULL, "script to execute by the admin plugin", "<filename>" },
 		
 		{ NULL,                       0, 0, G_OPTION_ARG_NONE,   NULL, NULL, NULL }
@@ -555,8 +564,22 @@ static int network_mysqld_admin_plugin_apply_config(chassis *chas, chassis_plugi
 	network_socket *listen_sock;
 
 	if (!config->address) config->address = g_strdup(":4041");
-	if (!config->admin_username) config->admin_username = g_strdup("root");
-	if (!config->admin_password) config->admin_password = g_strdup("secret");
+	if (!config->admin_username) {
+		g_critical("%s: --admin-username needs to be set",
+				G_STRLOC);
+		return -1;
+	}
+	if (!config->admin_password) {
+		g_critical("%s: --admin-password needs to be set",
+				G_STRLOC);
+		return -1;
+	}
+	if (!config->lua_script) {
+		g_critical("%s: --admin-lua-script needs to be set, <install-dir>/lib/mysql-proxy/lua/admin.lua may be a good value",
+				G_STRLOC);
+		return -1;
+	}
+
 
 	/** 
 	 * create a connection handle for the listen socket 
@@ -596,7 +619,7 @@ static int network_mysqld_admin_plugin_apply_config(chassis *chas, chassis_plugi
 G_MODULE_EXPORT int plugin_init(chassis_plugin *p) {
 	p->magic        = CHASSIS_PLUGIN_MAGIC;
 	p->name         = g_strdup("admin");
-	p->version		= g_strdup("0.7.0");
+	p->version		= g_strdup(PACKAGE_VERSION);
 
 	p->init         = network_mysqld_admin_plugin_new;
 	p->get_options  = network_mysqld_admin_plugin_get_options;
