@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
  
@@ -145,6 +147,7 @@ typedef int socklen_t;
 #include "lua-load-factory.h"
 
 #include "chassis-timings.h"
+#include "chassis-gtimeval.h"
 
 #define C(x) x, sizeof(x) - 1
 #define S(x) x->str, x->len
@@ -640,6 +643,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
 	if (!(auth->capabilities & CLIENT_PROTOCOL_41)) {
 		/* should use packet-id 0 */
 		network_mysqld_queue_append(con->client, con->client->send_queue, C("\xff\xd7\x07" "4.0 protocol is not supported"));
+		network_mysqld_auth_response_free(auth);
 		return NETWORK_SOCKET_ERROR;
 	}
 
@@ -944,36 +948,11 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth_result) {
 
 static network_mysqld_lua_stmt_ret proxy_lua_read_query(network_mysqld_con *con) {
 	network_mysqld_con_lua_t *st = con->plugin_con_state;
-	char command = -1;
 	network_socket *recv_sock = con->client;
 	GList   *chunk  = recv_sock->recv_queue->chunks->head;
 	GString *packet = chunk->data;
 	chassis_plugin_config *config = con->config;
-
-	if (!config->profiling) return PROXY_SEND_QUERY;
-
-	if (packet->len <= NET_HEADER_SIZE) return PROXY_SEND_QUERY; /* packet too short or zero-length packet */
 	
-	if (con->in_load_data_local_state) return PROXY_SEND_QUERY; /* don't try to intercept LOAD DATA LOCAL data packets */
-
-	command = packet->str[NET_HEADER_SIZE + 0];
-
-	if (COM_QUERY == command) {
-		/* we need some more data after the COM_QUERY */
-		if (packet->len < NET_HEADER_SIZE + 2) return PROXY_SEND_QUERY;
-
-		/* LOAD DATA INFILE is nasty */
-		if (packet->len - NET_HEADER_SIZE - 1 >= sizeof("LOAD ") - 1 &&
-		    0 == g_ascii_strncasecmp(packet->str + NET_HEADER_SIZE + 1, C("LOAD "))) return PROXY_SEND_QUERY;
-
-		/* don't cover them with injected queries as it trashes the result */
-		if (packet->len - NET_HEADER_SIZE - 1 >= sizeof("SHOW ERRORS") - 1 &&
-		    0 == g_ascii_strncasecmp(packet->str + NET_HEADER_SIZE + 1, C("SHOW ERRORS"))) return PROXY_SEND_QUERY;
-		if (packet->len - NET_HEADER_SIZE - 1 >= sizeof("select @@error_count") - 1 &&
-		    0 == g_ascii_strncasecmp(packet->str + NET_HEADER_SIZE + 1, C("select @@error_count"))) return PROXY_SEND_QUERY;
-	
-	}
-
 	network_injection_queue_reset(st->injected.queries);
 
 	/* ok, here we go */
@@ -1061,9 +1040,19 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query(network_mysqld_con *con)
 	
 				break;
 			case PROXY_NO_DECISION:
-				/**
-				 * PROXY_NO_DECISION and PROXY_SEND_QUERY may pick another backend
+				/* send on the data we got from the client unchanged
 				 */
+
+				if (st->injected.queries->length) {
+					injection *inj;
+
+					g_critical("%s: proxy.queue:append() or :prepend() used without 'return proxy.PROXY_SEND_QUERY'. Discarding %d elements from the queue.",
+							G_STRLOC,
+							st->injected.queries->length);
+
+					while ((inj = g_queue_pop_head(st->injected.queries))) injection_free(inj);
+				}
+			
 				break;
 			case PROXY_SEND_QUERY:
 				/* send the injected queries
@@ -1072,7 +1061,10 @@ static network_mysqld_lua_stmt_ret proxy_lua_read_query(network_mysqld_con *con)
 				 * 
 				 *  */
 
-				if (st->injected.queries->length) {
+				if (st->injected.queries->length == 0) {
+					g_critical("%s: 'return proxy.PROXY_SEND_QUERY' used without proxy.queue:append() or :prepend(). Assuming 'nil' was returned",
+							G_STRLOC);
+				} else {
 					ret = PROXY_SEND_INJECTION;
 				}
 	
@@ -1131,11 +1123,11 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query) {
 		return NETWORK_SOCKET_ERROR;
 	}
 	
-	send_sock = con->server;
-
 	switch (ret) {
 	case PROXY_NO_DECISION:
 	case PROXY_SEND_QUERY:
+		send_sock = con->server;
+
 		/* no injection, pass on the chunks as is */
 		while ((packet = g_queue_pop_head(recv_sock->recv_queue->chunks))) {
 			network_mysqld_queue_append_raw(send_sock, send_sock->send_queue, packet);
@@ -1143,18 +1135,41 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query) {
 		con->resultset_is_needed = FALSE; /* we don't want to buffer the result-set */
 
 		break;
-	case PROXY_SEND_RESULT: 
+	case PROXY_SEND_RESULT: {
+		gboolean is_first_packet = TRUE;
 		proxy_query = 0;
-		
-		while ((packet = g_queue_pop_head(recv_sock->recv_queue->chunks))) g_string_free(packet, TRUE);
 
-		break; 
+		send_sock = con->client;
+
+		/* flush the recv-queue and track the command-states */
+		while ((packet = g_queue_pop_head(recv_sock->recv_queue->chunks))) {
+			if (is_first_packet) {
+				network_packet p;
+
+				p.data = packet;
+				p.offset = 0;
+
+				network_mysqld_con_reset_command_response_state(con);
+
+				if (0 != network_mysqld_con_command_states_init(con, &p)) {
+					g_debug("%s: ", G_STRLOC);
+				}
+
+				is_first_packet = FALSE;
+			}
+
+			g_string_free(packet, TRUE);
+		}
+
+		break; }
 	case PROXY_SEND_INJECTION: {
 		injection *inj;
 
 		inj = g_queue_peek_head(st->injected.queries);
 		con->resultset_is_needed = inj->resultset_is_needed; /* let the lua-layer decide if we want to buffer the result or not */
 
+		send_sock = con->server;
+
 		network_mysqld_queue_reset(send_sock);
 		network_mysqld_queue_append(send_sock, send_sock->send_queue, S(inj->query));
 
@@ -1168,6 +1183,21 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query) {
 	if (proxy_query) {
 		con->state = CON_STATE_SEND_QUERY;
 	} else {
+		GList *cur;
+
+		/* if we don't send the query to the backend, it won't be tracked. So track it here instead 
+		 * to get the packet tracking right (LOAD DATA LOCAL INFILE, ...) */
+
+		for (cur = send_sock->send_queue->chunks->head; cur; cur = cur->next) {
+			network_packet p;
+			int r;
+
+			p.data = cur->data;
+			p.offset = 0;
+
+			r = network_mysqld_proto_get_query_result(&p, con);
+		}
+
 		con->state = CON_STATE_SEND_QUERY_RESULT;
 		con->resultset_is_finished = TRUE; /* we don't have more too send */
 	}
@@ -1217,6 +1247,8 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_send_query_result) {
 	}
 
 	if (st->injected.queries->length == 0) {
+		/* we have nothing more to send, let's see what the next state is */
+
 		con->state = CON_STATE_READ_QUERY;
 
 		return NETWORK_SOCKET_SUCCESS;
@@ -1299,6 +1331,8 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query_result) {
 	}
 
 	if (is_finished) {
+		network_mysqld_lua_stmt_ret ret;
+
 		/**
 		 * the resultset handler might decide to trash the send-queue
 		 * 
@@ -1326,14 +1360,16 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query_result) {
 			/* g_get_current_time(&(inj->ts_read_query_result_last)); */
 		}
 		
-		network_mysqld_queue_reset(recv_sock);
+		network_mysqld_queue_reset(recv_sock); /* reset the packet-id checks as the server-side is finished */
 
 		NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query_result::enter_lua");
-		proxy_lua_read_query_result(con);
+		ret = proxy_lua_read_query_result(con);
 		NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query_result::leave_lua");
 
-		/** recv_sock might be != con->server now */
-		network_mysqld_queue_reset(send_sock);
+		if (PROXY_IGNORE_RESULT != ret) {
+			/* reset the packet-id checks, if we sent something to the client */
+			network_mysqld_queue_reset(send_sock);
+		}
 
 		/**
 		 * if the send-queue is empty, we have nothing to send
@@ -1468,7 +1504,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 
 			/* mark the backend as being DOWN and retry with a different one */
 			st->backend->state = BACKEND_STATE_DOWN;
-			g_get_current_time(&(st->backend->state_since));
+			chassis_gtime_testset_now(&st->backend->state_since, NULL);
 			network_socket_free(con->server);
 			con->server = NULL;
 
@@ -1480,7 +1516,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 
 		if (st->backend->state != BACKEND_STATE_UP) {
 			st->backend->state = BACKEND_STATE_UP;
-			g_get_current_time(&(st->backend->state_since));
+			chassis_gtime_testset_now(&st->backend->state_since, NULL);
 		}
 
 		con->state = CON_STATE_READ_HANDSHAKE;
@@ -1572,7 +1608,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 	}
 
 	if (NULL == st->backend) {
-		network_mysqld_con_send_error(con->client, C("(proxy) all backends are down"));
+		network_mysqld_con_send_error_pre41(con->client, C("(proxy) all backends are down"));
 		g_critical("%s.%d: Cannot connect, all backends are down.", __FILE__, __LINE__);
 		return NETWORK_SOCKET_ERROR;
 	}
@@ -1598,7 +1634,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 					__FILE__, __LINE__, con->server->dst->name->str);
 
 			st->backend->state = BACKEND_STATE_DOWN;
-			g_get_current_time(&(st->backend->state_since));
+			chassis_gtime_testset_now(&st->backend->state_since, NULL);
 
 			network_socket_free(con->server);
 			con->server = NULL;
@@ -1608,7 +1644,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 
 		if (st->backend->state != BACKEND_STATE_UP) {
 			st->backend->state = BACKEND_STATE_UP;
-			g_get_current_time(&(st->backend->state_since));
+			chassis_gtime_testset_now(&st->backend->state_since, NULL);
 		}
 
 		con->state = CON_STATE_READ_HANDSHAKE;
@@ -1788,6 +1824,115 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_disconnect_client) {
 	return NETWORK_SOCKET_SUCCESS;
 }
 
+/**
+ * read the load data infile data from the client
+ *
+ * - decode the result-set to track if we are finished already
+ * - gets called once for each packet
+ *
+ * @FIXME stream the data to the backend
+ */
+NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_local_infile_data) {
+	int query_result = 0;
+	network_packet packet;
+	network_socket *recv_sock, *send_sock;
+	network_mysqld_com_query_result_t *com_query = con->parse.data;
+
+	NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query_result::enter");
+	
+	recv_sock = con->client;
+	send_sock = con->server;
+
+	/* check if the last packet is valid */
+	packet.data = g_queue_peek_tail(recv_sock->recv_queue->chunks);
+	packet.offset = 0;
+
+	/* if we get here from another state, src/network-mysqld.c is broken */
+	g_assert_cmpint(con->parse.command, ==, COM_QUERY);
+	g_assert_cmpint(com_query->state, ==, PARSE_COM_QUERY_LOCAL_INFILE_DATA);
+
+	query_result = network_mysqld_proto_get_query_result(&packet, con);
+	if (query_result == -1) return NETWORK_SOCKET_ERROR; /* something happend, let's get out of here */
+
+	if (con->server) {
+		network_mysqld_queue_append_raw(send_sock, send_sock->send_queue,
+				g_queue_pop_tail(recv_sock->recv_queue->chunks));
+	} else {
+		GString *s;
+		/* we don't have a backend
+		 *
+		 * - free the received packets early
+		 * - send a OK later 
+		 */
+		while ((s = g_queue_pop_head(recv_sock->recv_queue->chunks))) g_string_free(s, TRUE);
+	}
+
+	if (query_result == 1) { /* we have everything, send it to the backend */
+		if (con->server) {
+			con->state = CON_STATE_SEND_LOCAL_INFILE_DATA;
+		} else {
+			network_mysqld_con_send_ok(con->client);
+			con->state = CON_STATE_SEND_LOCAL_INFILE_RESULT;
+		}
+		g_assert_cmpint(com_query->state, ==, PARSE_COM_QUERY_LOCAL_INFILE_RESULT);
+	}
+
+	return NETWORK_SOCKET_SUCCESS;
+}
+
+/**
+ * read the load data infile result from the server
+ *
+ * - decode the result-set to track if we are finished already
+ */
+NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_local_infile_result) {
+	int query_result = 0;
+	network_packet packet;
+	network_socket *recv_sock, *send_sock;
+
+	NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_local_infile_result::enter");
+
+	recv_sock = con->server;
+	send_sock = con->client;
+
+	/* check if the last packet is valid */
+	packet.data = g_queue_peek_tail(recv_sock->recv_queue->chunks);
+	packet.offset = 0;
+	
+	query_result = network_mysqld_proto_get_query_result(&packet, con);
+	if (query_result == -1) return NETWORK_SOCKET_ERROR; /* something happend, let's get out of here */
+
+	network_mysqld_queue_append_raw(send_sock, send_sock->send_queue,
+			g_queue_pop_tail(recv_sock->recv_queue->chunks));
+
+	if (query_result == 1) {
+		con->state = CON_STATE_SEND_LOCAL_INFILE_RESULT;
+	}
+
+	return NETWORK_SOCKET_SUCCESS;
+}
+
+/**
+ * cleanup after we sent to result of the LOAD DATA INFILE LOCAL data to the client
+ */
+NETWORK_MYSQLD_PLUGIN_PROTO(proxy_send_local_infile_result) {
+	network_socket *recv_sock, *send_sock;
+
+	NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::send_local_infile_result::enter");
+
+	recv_sock = con->server;
+	send_sock = con->client;
+
+	/* reset the packet-ids */
+	if (send_sock) network_mysqld_queue_reset(send_sock);
+	if (recv_sock) network_mysqld_queue_reset(recv_sock);
+
+	con->state = CON_STATE_READ_QUERY;
+
+	return NETWORK_SOCKET_SUCCESS;
+}
+
+
 int network_mysqld_proxy_connection_init(network_mysqld_con *con) {
 	con->plugins.con_init                      = proxy_init;
 	con->plugins.con_connect_server            = proxy_connect_server;
@@ -1797,6 +1942,9 @@ int network_mysqld_proxy_connection_init(network_mysqld_con *con) {
 	con->plugins.con_read_query                = proxy_read_query;
 	con->plugins.con_read_query_result         = proxy_read_query_result;
 	con->plugins.con_send_query_result         = proxy_send_query_result;
+	con->plugins.con_read_local_infile_data = proxy_read_local_infile_data;
+	con->plugins.con_read_local_infile_result = proxy_read_local_infile_result;
+	con->plugins.con_send_local_infile_result = proxy_send_local_infile_result;
 	con->plugins.con_cleanup                   = proxy_disconnect_client;
 
 	return 0;
@@ -1930,23 +2078,27 @@ int network_mysqld_proxy_plugin_apply_config(chassis *chas, chassis_plugin_confi
 	/* set the plugin hooks as we want to apply them to the new connections too later */
 	network_mysqld_proxy_connection_init(con);
 
-	/* FIXME: network_socket_set_address() */
 	if (0 != network_address_set_address(listen_sock->dst, config->address)) {
 		return -1;
 	}
 
-	/* FIXME: network_socket_bind() */
 	if (0 != network_socket_bind(listen_sock)) {
 		return -1;
 	}
 	g_message("proxy listening on port %s", config->address);
 
 	for (i = 0; config->backend_addresses && config->backend_addresses[i]; i++) {
-		network_backends_add(g->backends, config->backend_addresses[i], BACKEND_TYPE_RW);
+		if (-1 == network_backends_add(g->backends, config->backend_addresses[i],
+				BACKEND_TYPE_RW)) {
+			return -1;
+		}
 	}
 	
 	for (i = 0; config->read_only_backend_addresses && config->read_only_backend_addresses[i]; i++) {
-		network_backends_add(g->backends, config->read_only_backend_addresses[i], BACKEND_TYPE_RO);
+		if (-1 == network_backends_add(g->backends,
+				config->read_only_backend_addresses[i], BACKEND_TYPE_RO)) {
+			return -1;
+		}
 	}
 
 	/* load the script and setup the global tables */
@@ -1966,7 +2118,7 @@ int network_mysqld_proxy_plugin_apply_config(chassis *chas, chassis_plugin_confi
 G_MODULE_EXPORT int plugin_init(chassis_plugin *p) {
 	p->magic        = CHASSIS_PLUGIN_MAGIC;
 	p->name         = g_strdup("proxy");
-	p->version		= g_strdup("0.7.0");
+	p->version		= g_strdup(PACKAGE_VERSION);
 
 	p->init         = network_mysqld_proxy_plugin_new;
 	p->get_options  = network_mysqld_proxy_plugin_get_options;
