@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
  
@@ -27,6 +29,11 @@
 
 #include "chassis-exports.h"
 
+#define CHASSIS_RESOLUTION_SEC	0x0
+#define CHASSIS_RESOLUTION_MS	0x1
+
+#define CHASSIS_RESOLUTION_DEFAULT	CHASSIS_RESOLUTION_SEC
+
 /** @addtogroup chassis */
 /*@{*/
 typedef struct {
@@ -44,6 +51,7 @@ typedef struct {
 	gboolean rotate_logs;
 
 	GString *log_ts_str;
+	gint	 log_ts_resolution;	/*<< timestamp resolution (sec, ms) */
 
 	GString *last_msg;
 	time_t   last_msg_ts;
@@ -58,6 +66,10 @@ CHASSIS_API void chassis_log_free(chassis_log *log);
 CHASSIS_API int chassis_log_open(chassis_log *log);
 CHASSIS_API void chassis_log_func(const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data);
 CHASSIS_API void chassis_log_set_logrotate(chassis_log *log);
+CHASSIS_API int chassis_log_set_event_log(chassis_log *log, const char *app_name);
+CHASSIS_API const char *chassis_log_skip_topsrcdir(const char *message);
+CHASSIS_API void chassis_set_logtimestamp_resolution(chassis_log *log, int res);
+CHASSIS_API int chassis_get_logtimestamp_resolution(chassis_log *log);
 /*@}*/
 
 #endif
