@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
  
@@ -116,6 +118,8 @@ chassis *chassis_new() {
 
 	chas->event_hdr_version = g_strdup(_EVENT_VERSION);
 
+	chas->shutdown_hooks = chassis_shutdown_hooks_new();
+
 	return chas;
 }
 
@@ -137,6 +141,7 @@ void chassis_free(chassis *chas) {
 
 	/* init the shutdown, without freeing share structures */	
 	if (chas->priv_shutdown) chas->priv_shutdown(chas, chas->priv);
+	
 
 	/* call the destructor for all plugins */
 	for (i = 0; i < chas->modules->len; i++) {
@@ -144,6 +149,13 @@ void chassis_free(chassis *chas) {
 
 		g_assert(p->destroy);
 		p->destroy(p->config);
+	}
+	
+	chassis_shutdown_hooks_call(chas->shutdown_hooks); /* cleanup the global 3rd party stuff before we unload the modules */
+
+	for (i = 0; i < chas->modules->len; i++) {
+		chassis_plugin *p = chas->modules->pdata[i];
+
 		chassis_plugin_free(p);
 	}
 	
@@ -175,6 +187,8 @@ void chassis_free(chassis *chas) {
 	}
 #endif
 	g_free(chas->event_hdr_version);
+
+	chassis_shutdown_hooks_free(chas->shutdown_hooks);
 	
 	g_free(chas);
 }
@@ -195,11 +209,11 @@ static void sigterm_handler(int G_GNUC_UNUSED fd, short G_GNUC_UNUSED event_type
 static void sighup_handler(int G_GNUC_UNUSED fd, short G_GNUC_UNUSED event_type, void *_data) {
 	chassis *chas = _data;
 
-	g_message("received a SIGHUP, rotating logfile"); /* this should go into the old logfile */
+	g_message("received a SIGHUP, closing log file"); /* this should go into the old logfile */
 
 	chassis_log_set_logrotate(chas->log);
 	
-	g_message("rotated logfile"); /* ... and this into the new one */
+	g_message("re-opened log file after SIGHUP"); /* ... and this into the new one */
 }
 
 
@@ -249,6 +263,8 @@ int chassis_mainloop(void *_chas) {
 
 		g_assert(p->apply_config);
 		if (0 != p->apply_config(chas, p->config)) {
+			g_critical("%s: applying config of plugin %s failed",
+					G_STRLOC, p->name);
 			return -1;
 		}
 	}
