@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
  
@@ -40,6 +42,7 @@
 #include "chassis-exports.h"
 #include "chassis-log.h"
 #include "chassis-stats.h"
+#include "chassis-shutdown-hooks.h"
 
 /** @defgroup chassis Chassis
  * 
@@ -72,6 +75,8 @@ struct chassis {
 	gint event_thread_count;
 
 	chassis_event_threads_t *threads;
+
+	chassis_shutdown_hooks_t *shutdown_hooks;
 };
 
 CHASSIS_API chassis *chassis_init(void) G_GNUC_DEPRECATED;
