@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
  
@@ -26,8 +28,8 @@
 #include "chassis-mainloop.h"
 #include "chassis-exports.h"
 
-/* current magic is 0.7.0-4 */
-#define CHASSIS_PLUGIN_MAGIC 0x00070004L
+/* current magic is 0.8.0-1 */
+#define CHASSIS_PLUGIN_MAGIC 0x00080001L
 
 /**
  * The private stats structure of a plugin. This is opaque to the rest of the code,
@@ -40,7 +42,8 @@ typedef struct chassis_plugin_config chassis_plugin_config;
 typedef struct chassis_plugin {
 	long      magic;    /**< a magic token to verify that the plugin API matches */
 
-	gchar    *name;     /**< the name of the plugin as defined */
+	gchar    *option_grp_name;     /**< name of the option group (used in --help-<option-grp> */
+	gchar    *name;     /**< user visible name of this plugin */
 	gchar    *version;  /**< the plugin's version number */
 	GModule  *module;   /**< the plugin handle when loaded */
 	
