@@ -1,22 +1,22 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2007-2009 MySQL AB, 2009 Sun Microsystems, Inc
+ Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
- 
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -45,6 +45,8 @@
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <glib.h>
+#include <glib/gstdio.h>
 
 #include "network-address.h"
 #include "glib-ext.h"
@@ -63,10 +65,36 @@ network_address *network_address_new() {
 }
 
 void network_address_free(network_address *addr) {
+
 	if (!addr) return;
 
-	g_string_free(addr->name, TRUE);
+#ifndef WIN32
+	/*
+	 * if the name we're freeing starts with a '/', we're
+	 * looking at a unix socket which needs to be removed
+	 */
+	if (addr->can_unlink_socket == TRUE && addr->name != NULL &&
+			addr->name->str != NULL) {
+		gchar	*name;
+		int		ret;
 
+		name = addr->name->str;
+		if (name[0] == '/') {
+			ret = g_remove(name);
+			if (ret == 0) {
+				g_debug("%s: removing socket %s successful", 
+					G_STRLOC, name);
+			} else {
+				if (errno != EPERM && errno != EACCES) {
+					g_critical("%s: removing socket %s failed: %s (%d)", 
+						G_STRLOC, name, strerror(errno), errno);
+				}
+			}
+		}
+	}
+#endif /* WIN32 */
+
+	g_string_free(addr->name, TRUE);
 	g_free(addr);
 }
 
@@ -90,12 +118,62 @@ static gint network_address_set_address_ip(network_address *addr, const gchar *a
 	    0 == strcmp("0.0.0.0", address)) {
 		/* no ip */
 		addr->addr.ipv4.sin_addr.s_addr = htonl(INADDR_ANY);
+		addr->addr.ipv4.sin_family = AF_INET; /* "default" family */
 	} else {
-		struct hostent *he;
+#ifdef HAVE_GETADDRINFO
+		struct addrinfo *ai = NULL, hint;
+		int				rc, family;
+		
+		memset(&hint, 0, sizeof (hint));
+		/*
+		 * FIXME: when we add support for IPv6, we'll have to do one
+		 * PF_INET* after the other
+		 */
+		hint.ai_family = PF_INET;
+		if ((rc = getaddrinfo(address, NULL, &hint, &ai)) != 0) {
+			g_critical("getaddrinfo(%s) failed with %s", address, 
+					   gai_strerror(rc));
+			return -1;
+		}
+
+		do {
+			family = ai->ai_family;
+#if 0 /* keep this for when we do IPv6 */
+			if (family == PF_INET6) {
+				memcpy(&addr->addr.ipv6,
+						(struct sockaddr_in6 *) ai->ai_addr,
+						sizeof (addr->addr.ipv6));
+				break;
+			} 
+#endif /* 0 */
+			if (family == PF_INET) {
+				memcpy(&addr->addr.ipv4,
+						(struct sockaddr_in *) ai->ai_addr, 
+						sizeof (addr->addr.ipv4));
+				break;
+			}
+			ai = ai->ai_next;
+		} while (NULL != ai);
+
+		if (ai == NULL) {
+			/* the family we print here is the *last* ai's */
+			g_critical("address %s doesn't resolve to a valid/supported "
+					   "address family: %d expected, last found %d", address,
+					   PF_INET, family);
+			freeaddrinfo(ai);
+			return -1;
+		}
+
+		freeaddrinfo(ai);
+#else 
+		struct hostent	*he;
+		static GStaticMutex gh_mutex = G_STATIC_MUTEX_INIT;
+
+		g_static_mutex_lock(&gh_mutex);
 
 		he = gethostbyname(address);
-
-		if (NULL == he)  {
+		if (NULL == he) {
+			g_static_mutex_unlock(&gh_mutex);
 			return -1;
 		}
 
@@ -103,13 +181,15 @@ static gint network_address_set_address_ip(network_address *addr, const gchar *a
 		g_assert(he->h_length == sizeof(struct in_addr));
 
 		memcpy(&(addr->addr.ipv4.sin_addr.s_addr), he->h_addr_list[0], he->h_length);
+		g_static_mutex_unlock(&gh_mutex);
+		addr->addr.ipv4.sin_family = AF_INET;
+#endif /* HAVE_GETADDRINFO */
 	}
 
-	addr->addr.ipv4.sin_family = AF_INET;
 	addr->addr.ipv4.sin_port = htons(port);
 	addr->len = sizeof(struct sockaddr_in);
 
-	network_address_refresh_name(addr);
+	(void) network_address_refresh_name(addr);
 
 	return 0;
 }
@@ -158,7 +238,7 @@ gint network_address_set_address(network_address *addr, const gchar *address) {
 		return network_address_set_address_un(addr, address);
 	
 	if (NULL != (s = strchr(address, ':'))) {
-		gboolean ret;
+		gint ret;
 		char *ip_address = g_strndup(address, s - address); /* may be NULL for strdup(..., 0) */
 		char *port_err = NULL;
 
@@ -218,6 +298,17 @@ gint network_address_refresh_name(network_address *addr) {
  */
 gboolean network_address_is_local(network_address *dst_addr, network_address *src_addr) {
 	if (src_addr->addr.common.sa_family != dst_addr->addr.common.sa_family) {
+#ifdef HAVE_SYS_UN_H
+		if (src_addr->addr.common.sa_family == AF_UNIX ||
+		    dst_addr->addr.common.sa_family == AF_UNIX) {
+			/* AF_UNIX is always local,
+			 * even if one of the two sides doesn't return a reasonable protocol 
+			 *
+			 * see #42220
+			 */
+			return TRUE;
+		}
+#endif
 		g_message("%s: is-local family %d != %d",
 				G_STRLOC,
 				src_addr->addr.common.sa_family,
