@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
 /**
@@ -45,24 +47,16 @@ void network_mysqld_com_query_result_free(network_mysqld_com_query_result_t *uda
 	g_free(udata);
 }
 
-/* TODO: is this one dead? */
-int network_mysqld_com_query_result_track_state(network_packet *packet, network_mysqld_com_query_result_t *udata) {
-	int err = 0;
-
-	if (udata->state == PARSE_COM_QUERY_LOAD_DATA) {
-		/* if the packet length is 0, the client is done */
-		guint32 len;
-
-	       	err = err || network_mysqld_proto_get_int24(packet, &len);
-
-		if (len == 0) {
-			udata->state = PARSE_COM_QUERY_LOAD_DATA_END_DATA;
-		}
-	}
-
-	return err ? -1 : 0;
+/**
+ * unused
+ *
+ * @deprecated will be removed in 0.9
+ * @see network_mysqld_proto_get_com_query_result
+ */
+int network_mysqld_com_query_result_track_state(network_packet G_GNUC_UNUSED *packet, network_mysqld_com_query_result_t G_GNUC_UNUSED *udata) {
+	g_error("%s: this function is deprecated and network_mysqld_proto_get_com_query_result() should be used instead",
+			G_STRLOC);
 }
-
 /**
  * @return -1 on error
  *         0  on success and done
@@ -115,7 +109,7 @@ int network_mysqld_proto_get_com_query_result(network_packet *packet, network_my
 			break;
 		case MYSQLD_PACKET_NULL:
 			/* OH NO, LOAD DATA INFILE :) */
-			query->state = PARSE_COM_QUERY_LOAD_DATA;
+			query->state = PARSE_COM_QUERY_LOCAL_INFILE_DATA;
 			is_finished = 1;
 
 			break;
@@ -231,7 +225,14 @@ int network_mysqld_proto_get_com_query_result(network_packet *packet, network_my
 			break;
 		}
 		break;
-	case PARSE_COM_QUERY_LOAD_DATA_END_DATA:
+	case PARSE_COM_QUERY_LOCAL_INFILE_DATA: 
+		/* we will receive a empty packet if we are done */
+		if (packet->data->len == packet->offset) {
+			query->state = PARSE_COM_QUERY_LOCAL_INFILE_RESULT;
+			is_finished = 1;
+		}
+		break;
+	case PARSE_COM_QUERY_LOCAL_INFILE_RESULT:
 		err = err || network_mysqld_proto_get_int8(packet, &status);
 		if (err) break;
 
@@ -252,12 +253,6 @@ int network_mysqld_proto_get_com_query_result(network_packet *packet, network_my
 			break;
 		}
 
-		break;
-	default:
-		g_critical("%s: unknown state in COM_QUERY: %d", 
-				G_STRLOC,
-				query->state);
-		err = 1;
 		break;
 	}
 
@@ -266,9 +261,24 @@ int network_mysqld_proto_get_com_query_result(network_packet *packet, network_my
 	return is_finished;
 }
 
-
+/**
+ * check if the we are in the LOCAL INFILE 'send data from client' state
+ *
+ * is deprecated as the name doesn't reflect its purpose:
+ * - it isn't triggered for LOAD DATA INFILE (w/o LOCAL)
+ * - it also covers LOAD XML LOCAL INFILE
+ *
+ * @deprecated use network_mysqld_com_query_result_is_local_infile() instead
+ */
 gboolean network_mysqld_com_query_result_is_load_data(network_mysqld_com_query_result_t *udata) {
-	return (udata->state == PARSE_COM_QUERY_LOAD_DATA) ? TRUE : FALSE;
+	return network_mysqld_com_query_result_is_local_infile(udata);
+}
+
+/**
+ * check if the we are in the LOCAL INFILE 'send data from client' state
+ */
+gboolean network_mysqld_com_query_result_is_local_infile(network_mysqld_com_query_result_t *udata) {
+	return (udata->state == PARSE_COM_QUERY_LOCAL_INFILE_DATA) ? TRUE : FALSE;
 }
 
 network_mysqld_com_stmt_prepare_result_t *network_mysqld_com_stmt_prepare_result_new() {
@@ -407,12 +417,14 @@ int network_mysqld_proto_get_com_init_db(
 	case MYSQLD_PACKET_OK:
 		/**
 		 * track the change of the init_db */
-		g_string_truncate(con->server->default_db, 0);
+		if (con->server) g_string_truncate(con->server->default_db, 0);
 		g_string_truncate(con->client->default_db, 0);
 
 		if (udata->db_name && udata->db_name->len) {
-			g_string_append_len(con->server->default_db, 
-					S(udata->db_name));
+			if (con->server) {
+				g_string_append_len(con->server->default_db, 
+						S(udata->db_name));
+			}
 			
 			g_string_append_len(con->client->default_db, 
 					S(udata->db_name));
@@ -434,8 +446,60 @@ int network_mysqld_proto_get_com_init_db(
 }
 
 /**
- * @return -1 on error, 
- *          0 is not finished, 
+ * init the tracking of the sub-states of the protocol
+ */
+int network_mysqld_con_command_states_init(network_mysqld_con *con, network_packet *packet) {
+	guint8 cmd;
+	int err = 0;
+
+	err = err || network_mysqld_proto_skip_network_header(packet);
+	err = err || network_mysqld_proto_get_int8(packet, &cmd);
+
+	if (err) return -1;
+
+	con->parse.command = cmd;
+
+	packet->offset = 0; /* reset the offset again for the next functions */
+
+	/* init the parser for the commands */
+	switch (con->parse.command) {
+	case COM_QUERY:
+	case COM_PROCESS_INFO:
+	case COM_STMT_EXECUTE:
+		con->parse.data = network_mysqld_com_query_result_new();
+		con->parse.data_free = (GDestroyNotify)network_mysqld_com_query_result_free;
+		break;
+	case COM_STMT_PREPARE:
+		con->parse.data = network_mysqld_com_stmt_prepare_result_new();
+		con->parse.data_free = (GDestroyNotify)network_mysqld_com_stmt_prepare_result_free;
+		break;
+	case COM_INIT_DB:
+		con->parse.data = network_mysqld_com_init_db_result_new();
+		con->parse.data_free = (GDestroyNotify)network_mysqld_com_init_db_result_free;
+
+		network_mysqld_com_init_db_result_track_state(packet, con->parse.data);
+
+		break;
+	case COM_QUIT:
+		/* track COM_QUIT going to the server, to be able to tell if the server
+		 * a) simply went away or
+		 * b) closed the connection because the client asked it to
+		 * If b) we should not print a message at the next EV_READ event from the server fd
+		 */
+		con->com_quit_seen = TRUE;
+	default:
+		break;
+	}
+
+	return 0;
+}
+
+/**
+ * @param packet the current packet that is passing by
+ *
+ *
+ * @return -1 on invalid packet, 
+ *          0 need more packets, 
  *          1 for the last packet 
  */
 int network_mysqld_proto_get_query_result(network_packet *packet, network_mysqld_con *con) {
@@ -583,13 +647,6 @@ int network_mysqld_proto_get_query_result(network_packet *packet, network_mysqld
 	case COM_PROCESS_INFO:
 	case COM_QUERY:
 		is_finished = network_mysqld_proto_get_com_query_result(packet, con->parse.data, FALSE);
-		{
-			network_mysqld_com_query_result_t *query = (network_mysqld_com_query_result_t*)con->parse.data;
-
-			if (query->state == PARSE_COM_QUERY_LOAD_DATA) {
-				con->in_load_data_local_state = TRUE;
-			}
-		}
 		break;
 	case COM_BINLOG_DUMP:
 		/**
@@ -598,17 +655,6 @@ int network_mysqld_proto_get_query_result(network_packet *packet, network_mysqld
 		 */
 		is_finished = 1;
 		break;
-	case COM_SLEEP:
-		/** 
-		 * this is not COM_SLEEP, even though it shares the same numeric value.
-		 * when executing a LOAD DATA LOCAL INFILE the server simply responds with
-		 * an OK packet (5th byte is 0x00), seen here as COM_SLEEP.
-		 * That's ok, because COM_SLEEP is an internal state and does not go over the wire.
-		 */
-		err = err || network_mysqld_proto_peek_int8(packet, &status);
-		if (err) return -1;
-		is_finished = 1;
-		break;
 	default:
 		g_critical("%s: COM_(0x%02x) is not handled", 
 				G_STRLOC,
@@ -816,16 +862,25 @@ int network_mysqld_proto_append_ok_packet(GString *packet, network_mysqld_ok_pac
 	return 0;
 }
 
-network_mysqld_err_packet_t *network_mysqld_err_packet_new() {
+static network_mysqld_err_packet_t *network_mysqld_err_packet_new_full(network_mysqld_protocol_t version) {
 	network_mysqld_err_packet_t *err_packet;
 
 	err_packet = g_new0(network_mysqld_err_packet_t, 1);
 	err_packet->sqlstate = g_string_new(NULL);
 	err_packet->errmsg = g_string_new(NULL);
+	err_packet->version = version;
 
 	return err_packet;
 }
 
+network_mysqld_err_packet_t *network_mysqld_err_packet_new() {
+	return network_mysqld_err_packet_new_full(NETWORK_MYSQLD_PROTOCOL_VERSION_41);
+}
+
+network_mysqld_err_packet_t *network_mysqld_err_packet_new_pre41() {
+	return network_mysqld_err_packet_new_full(NETWORK_MYSQLD_PROTOCOL_VERSION_PRE41);
+}
+
 void network_mysqld_err_packet_free(network_mysqld_err_packet_t *err_packet) {
 	if (!err_packet) return;
 
@@ -892,11 +947,10 @@ int network_mysqld_proto_get_err_packet(network_packet *packet, network_mysqld_e
  */
 int network_mysqld_proto_append_err_packet(GString *packet, network_mysqld_err_packet_t *err_packet) {
 	int errmsg_len;
-	guint32 capabilities = CLIENT_PROTOCOL_41;
 
 	network_mysqld_proto_append_int8(packet, 0xff); /* ERR */
 	network_mysqld_proto_append_int16(packet, err_packet->errcode); /* errorcode */
-	if (capabilities & CLIENT_PROTOCOL_41) {
+	if (err_packet->version == NETWORK_MYSQLD_PROTOCOL_VERSION_41) {
 		g_string_append_c(packet, '#');
 		if (err_packet->sqlstate && (err_packet->sqlstate->len > 0)) {
 			g_string_append_len(packet, err_packet->sqlstate->str, 5);
