@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
  
@@ -61,6 +63,7 @@
 #include <mysql.h>
 #include <mysqld_error.h>
 
+#include "network-debug.h"
 #include "network-mysqld.h"
 #include "network-mysqld-proto.h"
 #include "network-mysqld-packet.h"
@@ -201,6 +204,7 @@ network_mysqld_con *network_mysqld_con_new() {
 
 	con = g_new0(network_mysqld_con, 1);
 	con->timestamps = chassis_timestamps_new();
+	con->parse.command = -1;
 
 	return con;
 }
@@ -421,26 +425,17 @@ int network_mysqld_con_send_ok(network_socket *con) {
 	return network_mysqld_con_send_ok_full(con, 0, 0, SERVER_STATUS_AUTOCOMMIT, 0);
 }
 
-/**
- * send a error packet to the client connection
- *
- * @note the sqlstate has to match the SQL standard. If no matching SQL state is known, leave it at NULL
- *
- * @param con         the client connection
- * @param errmsg      the error message
- * @param errmsg_len  byte-len of the error-message
- * @param errorcode   mysql error-code we want to send
- * @param sqlstate    if none-NULL, 5-char SQL state to send, if NULL, default SQL state is used
- *
- * @return 0 on success
- */
-int network_mysqld_con_send_error_full(network_socket *con, const char *errmsg, gsize errmsg_len, guint errorcode, const gchar *sqlstate) {
+static int network_mysqld_con_send_error_full_all(network_socket *con,
+		const char *errmsg, gsize errmsg_len,
+		guint errorcode,
+		const gchar *sqlstate,
+		gboolean is_41_protocol) {
 	GString *packet;
 	network_mysqld_err_packet_t *err_packet;
 
 	packet = g_string_sized_new(10 + errmsg_len);
 	
-	err_packet = network_mysqld_err_packet_new();
+	err_packet = is_41_protocol ? network_mysqld_err_packet_new() : network_mysqld_err_packet_new_pre41();
 	err_packet->errcode = errorcode;
 	if (errmsg) g_string_assign_len(err_packet->errmsg, errmsg, errmsg_len);
 	if (sqlstate) g_string_assign_len(err_packet->sqlstate, sqlstate, strlen(sqlstate));
@@ -456,6 +451,24 @@ int network_mysqld_con_send_error_full(network_socket *con, const char *errmsg,
 	return 0;
 }
 
+/**
+ * send a error packet to the client connection
+ *
+ * @note the sqlstate has to match the SQL standard. If no matching SQL state is known, leave it at NULL
+ *
+ * @param con         the client connection
+ * @param errmsg      the error message
+ * @param errmsg_len  byte-len of the error-message
+ * @param errorcode   mysql error-code we want to send
+ * @param sqlstate    if none-NULL, 5-char SQL state to send, if NULL, default SQL state is used
+ *
+ * @return 0 on success
+ */
+int network_mysqld_con_send_error_full(network_socket *con, const char *errmsg, gsize errmsg_len, guint errorcode, const gchar *sqlstate) {
+	return network_mysqld_con_send_error_full_all(con, errmsg, errmsg_len, errorcode, sqlstate, TRUE);
+}
+
+
 /**
  * send a error-packet to the client connection
  *
@@ -471,6 +484,34 @@ int network_mysqld_con_send_error(network_socket *con, const char *errmsg, gsize
 	return network_mysqld_con_send_error_full(con, errmsg, errmsg_len, ER_UNKNOWN_ERROR, NULL);
 }
 
+/**
+ * send a error packet to the client connection (pre-4.1 protocol)
+ *
+ * @param con         the client connection
+ * @param errmsg      the error message
+ * @param errmsg_len  byte-len of the error-message
+ * @param errorcode   mysql error-code we want to send
+ *
+ * @return 0 on success
+ */
+int network_mysqld_con_send_error_pre41_full(network_socket *con, const char *errmsg, gsize errmsg_len, guint errorcode) {
+	return network_mysqld_con_send_error_full_all(con, errmsg, errmsg_len, errorcode, NULL, FALSE);
+}
+
+/**
+ * send a error-packet to the client connection (pre-4.1 protocol)
+ *
+ * @param con         the client connection
+ * @param errmsg      the error message
+ * @param errmsg_len  byte-len of the error-message
+ *
+ * @see network_mysqld_con_send_error_pre41_full
+ */
+int network_mysqld_con_send_error_pre41(network_socket *con, const char *errmsg, gsize errmsg_len) {
+	return network_mysqld_con_send_error_pre41_full(con, errmsg, errmsg_len, ER_UNKNOWN_ERROR);
+}
+
+
 /**
  * get a full packet from the raw queue and move it to the packet queue 
  */
@@ -500,6 +541,11 @@ network_socket_retval_t network_mysqld_con_get_packet(chassis G_GNUC_UNUSED*chas
 
 	/* move the packet from the raw queue to the recv-queue */
 	if ((packet = network_queue_pop_string(con->recv_queue_raw, packet_len + NET_HEADER_SIZE, NULL))) {
+#ifdef NETWORK_DEBUG_TRACE_IO
+		/* to trace the data we received from the socket, enable this */
+		g_debug_hexdump(G_STRLOC, S(packet));
+#endif
+
 		if (con->packet_id_is_reset) {
 			con->last_packet_id = packet_id;
 			con->packet_id_is_reset = FALSE;
@@ -685,6 +731,39 @@ network_socket_retval_t plugin_call(chassis *srv, network_mysqld_con *con, int s
 			con->state = CON_STATE_READ_QUERY;
 		}
 		break;
+
+	case CON_STATE_SEND_LOCAL_INFILE_DATA:
+		func = con->plugins.con_send_local_infile_data;
+
+		if (!func) { /* default implementation */
+			con->state = CON_STATE_READ_LOCAL_INFILE_RESULT;
+		}
+
+		break;
+	case CON_STATE_READ_LOCAL_INFILE_DATA:
+		func = con->plugins.con_read_local_infile_data;
+
+		if (!func) { /* the plugins have to implement this function to track LOAD DATA LOCAL INFILE handling work */
+			con->state = CON_STATE_ERROR;
+		}
+
+		break;
+	case CON_STATE_SEND_LOCAL_INFILE_RESULT:
+		func = con->plugins.con_send_local_infile_result;
+
+		if (!func) { /* default implementation */
+			con->state = CON_STATE_READ_QUERY;
+		}
+
+		break;
+	case CON_STATE_READ_LOCAL_INFILE_RESULT:
+		func = con->plugins.con_read_local_infile_result;
+
+		if (!func) { /* the plugins have to implement this function to track LOAD DATA LOCAL INFILE handling work */
+			con->state = CON_STATE_ERROR;
+		}
+
+		break;
 	case CON_STATE_ERROR:
 		g_debug("%s.%d: not executing plugin function in state CON_STATE_ERROR", __FILE__, __LINE__);
 		return NETWORK_SOCKET_SUCCESS;
@@ -718,6 +797,37 @@ void network_mysqld_con_reset_command_response_state(network_mysqld_con *con) {
 	}
 }
 
+/**
+ * get the name of a connection state
+ */
+const char *network_mysqld_con_state_get_name(network_mysqld_con_state_t state) {
+	switch (state) {
+	case CON_STATE_INIT: return "CON_STATE_INIT";
+	case CON_STATE_CONNECT_SERVER: return "CON_STATE_CONNECT_SERVER";
+	case CON_STATE_READ_HANDSHAKE: return "CON_STATE_READ_HANDSHAKE";
+	case CON_STATE_SEND_HANDSHAKE: return "CON_STATE_SEND_HANDSHAKE";
+	case CON_STATE_READ_AUTH: return "CON_STATE_READ_AUTH";
+	case CON_STATE_SEND_AUTH: return "CON_STATE_SEND_AUTH";
+	case CON_STATE_READ_AUTH_OLD_PASSWORD: return "CON_STATE_READ_AUTH_OLD_PASSWORD";
+	case CON_STATE_SEND_AUTH_OLD_PASSWORD: return "CON_STATE_SEND_AUTH_OLD_PASSWORD";
+	case CON_STATE_READ_AUTH_RESULT: return "CON_STATE_READ_AUTH_RESULT";
+	case CON_STATE_SEND_AUTH_RESULT: return "CON_STATE_SEND_AUTH_RESULT";
+	case CON_STATE_READ_QUERY: return "CON_STATE_READ_QUERY";
+	case CON_STATE_SEND_QUERY: return "CON_STATE_SEND_QUERY";
+	case CON_STATE_READ_QUERY_RESULT: return "CON_STATE_READ_QUERY_RESULT";
+	case CON_STATE_SEND_QUERY_RESULT: return "CON_STATE_SEND_QUERY_RESULT";
+	case CON_STATE_READ_LOCAL_INFILE_DATA: return "CON_STATE_READ_LOCAL_INFILE_DATA";
+	case CON_STATE_SEND_LOCAL_INFILE_DATA: return "CON_STATE_SEND_LOCAL_INFILE_DATA";
+	case CON_STATE_READ_LOCAL_INFILE_RESULT: return "CON_STATE_READ_LOCAL_INFILE_RESULT";
+	case CON_STATE_SEND_LOCAL_INFILE_RESULT: return "CON_STATE_SEND_LOCAL_INFILE_RESULT";
+	case CON_STATE_CLOSE_CLIENT: return "CON_STATE_CLOSE_CLIENT";
+	case CON_STATE_CLOSE_SERVER: return "CON_STATE_CLOSE_SERVER";
+	case CON_STATE_ERROR: return "CON_STATE_ERROR";
+	case CON_STATE_SEND_ERROR: return "CON_STATE_SEND_ERROR";
+	}
+
+	return "unknown";
+}
 
 /**
  * handle the different states of the MySQL protocol
@@ -731,6 +841,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 	network_mysqld_con *con = user_data;
 	chassis *srv = con->srv;
 	int retval;
+	network_socket_retval_t call_ret;
 
 	g_assert(srv);
 	g_assert(con);
@@ -800,8 +911,17 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 	} else {
 		NETWORK_MYSQLD_CON_TRACK_TIME(con, "con_handle_start");
 	}
+
 	do {
 		ostate = con->state;
+#ifdef NETWORK_DEBUG_TRACE_STATE_CHANGES
+		/* if you need the state-change information without dtrace, enable this */
+		g_debug("%s: [%d] %s",
+				G_STRLOC,
+				getpid(),
+				network_mysqld_con_state_get_name(con->state));
+#endif
+
 		MYSQLPROXY_STATE_CHANGE(event_fd, events, con->state);
 		switch (con->state) {
 		case CON_STATE_ERROR:
@@ -1258,7 +1378,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				if (con->state != ostate) break; /* the state has changed (e.g. CON_STATE_ERROR) */
 
 				last_packet.data = g_queue_peek_tail(recv_sock->recv_queue->chunks);
-			} while (last_packet.data->len == PACKET_LEN_MAX + NET_HEADER_SIZE);
+			} while (last_packet.data->len == PACKET_LEN_MAX + NET_HEADER_SIZE); /* read all chunks of the overlong data */
 
 			if (con->server &&
 			    con->server->challenge &&
@@ -1296,63 +1416,45 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			}
 
-			network_mysqld_con_reset_command_response_state(con);
+			/**
+			 * there should be 3 possible next states from here:
+			 *
+			 * - CON_STATE_ERROR (if something went wrong and we want to close the connection
+			 * - CON_STATE_SEND_QUERY (if we want to send data to the con->server)
+			 * - CON_STATE_SEND_QUERY_RESULT (if we want to send data to the con->client)
+			 *
+			 * @todo verify this with a clean switch ()
+			 */
+
+			/* reset the tracked command
+			 *
+			 * if the plugin decided to send a result, it has to track the commands itself
+			 * otherwise LOAD DATA LOCAL INFILE and friends will fail
+			 */
+			if (con->state == CON_STATE_SEND_QUERY) {
+				network_mysqld_con_reset_command_response_state(con);
+			}
 
 			break; }
 		case CON_STATE_SEND_QUERY:
 			/* send the query to the server
 			 *
 			 * this state will loop until all the packets from the send-queue are flushed 
-			 *
-			 * we have to handle LOAD DATA INFILE LOCAL which can only happen for COM_QUERY packets
 			 */
 
 			if (con->server->send_queue->offset == 0) {
 				/* only parse the packets once */
 				network_packet packet;
-				GList *chunk;
 
-				g_assert(con->server->send_queue->chunks);
-				chunk = con->server->send_queue->chunks->head;
-
-				packet.data = chunk->data;
+				packet.data = g_queue_peek_head(con->server->send_queue->chunks);
 				packet.offset = 0;
 
-				/*  */
-				if (con->parse.command == COM_QUERY) {
-					network_mysqld_com_query_result_track_state(&packet, con->parse.data);
-				} else {
-					con->parse.command = packet.data->str[4];
-	
-					/* init the parser for the commands */
-					switch (con->parse.command) {
-					case COM_QUERY:
-					case COM_PROCESS_INFO:
-					case COM_STMT_EXECUTE:
-						con->parse.data = network_mysqld_com_query_result_new();
-						con->parse.data_free = (GDestroyNotify)network_mysqld_com_query_result_free;
-						break;
-					case COM_STMT_PREPARE:
-						con->parse.data = network_mysqld_com_stmt_prepare_result_new();
-						con->parse.data_free = (GDestroyNotify)network_mysqld_com_stmt_prepare_result_free;
-						break;
-					case COM_INIT_DB:
-						con->parse.data = network_mysqld_com_init_db_result_new();
-						con->parse.data_free = (GDestroyNotify)network_mysqld_com_init_db_result_free;
+				if (0 != network_mysqld_con_command_states_init(con, &packet)) {
+					g_debug("%s: tracking mysql protocol states failed",
+							G_STRLOC);
+					con->state = CON_STATE_ERROR;
 
-						network_mysqld_com_init_db_result_track_state(&packet, con->parse.data);
-
-						break;
-					case COM_QUIT:
-						/* track COM_QUIT going to the server, to be able to tell if the server
-						 a) simply went away or
-						 b) closed the connection because the client asked it to
-						 If b) we should not print a message at the next EV_READ event from the server fd
-						 */
-						con->com_quit_seen = TRUE;
-					default:
-						break;
-					}
+					break;
 				}
 			}
 	
@@ -1376,23 +1478,6 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			
 			if (con->state != ostate) break; /* the state has changed (e.g. CON_STATE_ERROR) */
 
-#if 0
-			/* in case we are waiting for LOAD DATA LOCAL INFILE data on this connection,
-			 we need to read the sentinel zero-length packet, too. Otherwise we will block
-			 this connection. Bug#37404
-			 */
-			if (con->in_load_data_local_state) {
-				/* the last packet of LOAD DATA LOCAL INFILE is zero-length, signalling "no more data following" */
-				if (con->parse.len == 0) {
-					con->state = CON_STATE_READ_QUERY_RESULT;
-					con->in_load_data_local_state = FALSE;
-				} else {
-					con->state = CON_STATE_READ_QUERY;
-				}
-				break;
-			}
-#endif
-
 			/* some statements don't have a server response */
 			switch (con->parse.command) {
 			case COM_STMT_SEND_LONG_DATA: /* not acked */
@@ -1401,13 +1486,6 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				if (con->client) network_mysqld_queue_reset(con->client);
 				if (con->server) network_mysqld_queue_reset(con->server);
 				break;
-			case COM_QUERY:
-				if (network_mysqld_com_query_result_is_load_data(con->parse.data)) {
-					con->state = CON_STATE_READ_QUERY;
-				} else {
-					con->state = CON_STATE_READ_QUERY_RESULT;
-				}
-				break;
 			default:
 				con->state = CON_STATE_READ_QUERY_RESULT;
 				break;
@@ -1504,6 +1582,175 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			}
 
+			/* special treatment for the LOAD DATA LOCAL INFILE command */
+			if (con->state != CON_STATE_ERROR &&
+			    con->parse.command == COM_QUERY &&
+			    1 == network_mysqld_com_query_result_is_local_infile(con->parse.data)) {
+				con->state = CON_STATE_READ_LOCAL_INFILE_DATA;
+			}
+
+			break;
+		case CON_STATE_READ_LOCAL_INFILE_DATA: {
+			/**
+			 * read the file content from the client 
+			 */
+			network_socket *recv_sock;
+
+			recv_sock = con->client;
+
+			/**
+			 * LDLI is usually a whole set of packets
+			 */
+			do {
+				switch (network_mysqld_read(srv, recv_sock)) {
+				case NETWORK_SOCKET_SUCCESS:
+					break;
+				case NETWORK_SOCKET_WAIT_FOR_EVENT:
+					/* call us again when you have a event */
+					WAIT_FOR_EVENT(recv_sock, EV_READ, NULL);
+					NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_load_infile_data");
+
+					return;
+				case NETWORK_SOCKET_ERROR_RETRY:
+				case NETWORK_SOCKET_ERROR:
+					g_critical("%s: network_mysqld_read(%s) returned an error",
+							G_STRLOC,
+							network_mysqld_con_state_get_name(ostate));
+					con->state = CON_STATE_ERROR;
+					break;
+				}
+
+				if (con->state != ostate) break; /* the state has changed (e.g. CON_STATE_ERROR) */
+
+				switch ((call_ret = plugin_call(srv, con, con->state))) {
+				case NETWORK_SOCKET_SUCCESS:
+					break;
+				default:
+					g_critical("%s: plugin_call(%s) unexpected return value: %d",
+							G_STRLOC,
+							network_mysqld_con_state_get_name(ostate),
+							call_ret);
+
+					con->state = CON_STATE_ERROR;
+					break;
+				}
+			} while (con->state == ostate); /* read packets from the network until the plugin decodes to go to the next state */
+	
+			break; }
+		case CON_STATE_SEND_LOCAL_INFILE_DATA: 
+			/* send the hand-shake to the client and wait for a response */
+
+			switch (network_mysqld_write(srv, con->server)) {
+			case NETWORK_SOCKET_SUCCESS:
+				break;
+			case NETWORK_SOCKET_WAIT_FOR_EVENT:
+				WAIT_FOR_EVENT(con->server, EV_WRITE, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_load_infile_data");
+				
+				return;
+			case NETWORK_SOCKET_ERROR_RETRY:
+			case NETWORK_SOCKET_ERROR:
+				/**
+				 * writing failed, closing connection
+				 */
+				con->state = CON_STATE_ERROR;
+				break;
+			}
+
+			if (con->state != ostate) break; /* the state has changed (e.g. CON_STATE_ERROR) */
+
+			switch ((call_ret = plugin_call(srv, con, con->state))) {
+			case NETWORK_SOCKET_SUCCESS:
+				break;
+			default:
+				g_critical("%s: plugin_call(%s) unexpected return value: %d",
+						G_STRLOC,
+						network_mysqld_con_state_get_name(ostate),
+						call_ret);
+
+				con->state = CON_STATE_ERROR;
+				break;
+			}
+
+			break;
+		case CON_STATE_READ_LOCAL_INFILE_RESULT: {
+			/**
+			 * read auth data from the remote mysql-server 
+			 */
+			network_socket *recv_sock;
+			recv_sock = con->server;
+			g_assert(events == 0 || event_fd == recv_sock->fd);
+
+			switch (network_mysqld_read(srv, recv_sock)) {
+			case NETWORK_SOCKET_SUCCESS:
+				break;
+			case NETWORK_SOCKET_WAIT_FOR_EVENT:
+				/* call us again when you have a event */
+				WAIT_FOR_EVENT(recv_sock, EV_READ, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_load_infile_result");
+
+				return;
+			case NETWORK_SOCKET_ERROR_RETRY:
+			case NETWORK_SOCKET_ERROR:
+				g_critical("%s: network_mysqld_read(%s) returned an error",
+						G_STRLOC,
+						network_mysqld_con_state_get_name(ostate));
+
+				con->state = CON_STATE_ERROR;
+				break;
+			}
+
+			if (con->state != ostate) break; /* the state has changed (e.g. CON_STATE_ERROR) */
+
+			switch ((call_ret = plugin_call(srv, con, con->state))) {
+			case NETWORK_SOCKET_SUCCESS:
+				break;
+			default:
+				g_critical("%s: plugin_call(%s) unexpected return value: %d",
+						G_STRLOC,
+						network_mysqld_con_state_get_name(ostate),
+						call_ret);
+
+				con->state = CON_STATE_ERROR;
+				break;
+			}
+	
+			break; }
+		case CON_STATE_SEND_LOCAL_INFILE_RESULT: 
+			/* send the hand-shake to the client and wait for a response */
+
+			switch (network_mysqld_write(srv, con->client)) {
+			case NETWORK_SOCKET_SUCCESS:
+				break;
+			case NETWORK_SOCKET_WAIT_FOR_EVENT:
+				WAIT_FOR_EVENT(con->client, EV_WRITE, NULL);
+				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_load_infile_result");
+				
+				return;
+			case NETWORK_SOCKET_ERROR_RETRY:
+			case NETWORK_SOCKET_ERROR:
+				/**
+				 * writing failed, closing connection
+				 */
+				con->state = CON_STATE_ERROR;
+				break;
+			}
+
+			if (con->state != ostate) break; /* the state has changed (e.g. CON_STATE_ERROR) */
+
+			switch ((call_ret = plugin_call(srv, con, con->state))) {
+			case NETWORK_SOCKET_SUCCESS:
+				break;
+			default:
+				g_critical("%s: plugin_call(%s) unexpected return value: %d",
+						G_STRLOC,
+						network_mysqld_con_state_get_name(ostate),
+						call_ret);
+
+				con->state = CON_STATE_ERROR;
+				break;
+			}
+
 			break;
 		case CON_STATE_SEND_ERROR:
 			/**
