@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
  
@@ -83,10 +85,12 @@
 #endif
 #endif
 
-
+#include "network-debug.h"
 #include "network-socket.h"
 #include "network-mysqld-proto.h"
 #include "network-mysqld-packet.h"
+#include "string-len.h"
+#include "glib-ext.h"
 
 #ifndef DISABLE_DEPRECATED_DECL
 network_socket *network_socket_init() {
@@ -427,6 +431,7 @@ network_socket_retval_t network_socket_bind(network_socket * con) {
 		}
 	}
 
+	con->dst->can_unlink_socket = TRUE;
 	return NETWORK_SOCKET_SUCCESS;
 }
 
@@ -447,7 +452,7 @@ network_socket_retval_t network_socket_read(network_socket *sock) {
 			len = recv(sock->fd, packet->str, sock->to_read, 0);
 		} else {
 			/* UDP */
-			socklen_t dst_len = sizeof(sock->dst->addr.common);
+			network_socklen_t dst_len = sizeof(sock->dst->addr.common);
 			len = recvfrom(sock->fd, packet->str, sock->to_read, 0, &(sock->dst->addr.common), &(dst_len));
 			sock->dst->len = dst_len;
 		}
@@ -575,7 +580,10 @@ static network_socket_retval_t network_socket_write_writev(network_socket *con,
 
 		if (con->send_queue->offset >= s->len) {
 			con->send_queue->offset -= s->len;
-
+#ifdef NETWORK_DEBUG_TRACE_IO
+			/* to trace the data we sent to the socket, enable this */
+			g_debug_hexdump(G_STRLOC, S(s));
+#endif
 			g_string_free(s, TRUE);
 			
 			g_queue_delete_link(con->send_queue->chunks, chunk);
@@ -682,7 +690,7 @@ network_socket_retval_t network_socket_to_read(network_socket *sock) {
 #ifdef SO_NREAD
 	/* on MacOS X ioctl(..., FIONREAD) returns _more_ than what we have in the queue */
 	if (sock->socket_type == SOCK_DGRAM) {
-		socklen_t b_len = sizeof(b);
+		network_socklen_t b_len = sizeof(b);
 
 		if (0 != getsockopt(sock->fd, SOL_SOCKET, SO_NREAD, &b, &b_len)) {
 			g_critical("%s: getsockopt(%d, SO_NREAD, ...) failed: %s (%d)",
