@@ -1,18 +1,20 @@
 --[[ $%BEGINLICENSE%$
- Copyright (C) 2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2008, 2009, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ --]]
 local chassis = require("chassis")
@@ -63,6 +65,32 @@ function read_query_result(inj)
 				}
 			}
 			return proxy.PROXY_SEND_RESULT
+		elseif inj.query == string.char(proxy.COM_QUERY) .. "SELECT \"1\", NULL, \"1\"" then
+			-- traverse the resultset using res.rows and check if the fields after the NULL field are accessible
+			assert(res.affected_rows == nil)
+
+			-- convert the MySQL row into a string
+			local test_res = ""
+			for row in res.rows do
+				test_res = ("%s,%s,%s"):format(
+					tostring(row[1]),
+					tostring(row[2]),
+					tostring(row[3]))
+			end
+
+			-- return the string to mysqltest (should be 1,nil,1)
+			proxy.response = {
+				type = proxy.MYSQLD_PACKET_OK,
+				resultset = {
+					fields = { 
+						{ name = "test_result" }
+					},
+					rows = {
+						{ test_res }
+					}
+				}
+			}
+			return proxy.PROXY_SEND_RESULT
 		elseif inj.query == string.char(proxy.COM_QUERY) .. "SELECT row_count(1), bytes()" then
 			-- convert a OK packet with affected rows into a resultset
 			assert(res.affected_rows == nil)
