@@ -1,18 +1,20 @@
 --[[ $%BEGINLICENSE%$
- Copyright (C) 2007-2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ --]]
 
@@ -27,6 +29,7 @@
 -- we require LFS (LuaFileSystem)
 require("lfs")
 require("glib2")
+require("posix")
 
 ---
 -- get the directory-name of a path
@@ -81,6 +84,7 @@ local MYSQL_PORT	 	= os.getenv("MYSQL_PORT")	 	or "3306"
 local MYSQL_DB	   		= os.getenv("MYSQL_DB")	   		or "test"
 local MYSQL_TEST_BIN 	= os.getenv("MYSQL_TEST_BIN") 	or "mysqltest"
 local MYSQL_CLIENT_BIN 	= os.getenv("MYSQL_CLIENT_BIN") or "mysql"
+local TESTS_REGEX 	= os.getenv("TESTS_REGEX")
 
 --
 -- Global variables that can be referenced from .options files
@@ -91,16 +95,11 @@ function get_port_base()
 	local port_base_end   = os.getenv("MYSQL_PROXY_END_PORT") or 65535
 	local port_interval   = 64 -- let's take the base port in steps of ...
 	local range = port_base_end - port_base_start - port_interval
-	print(os.time())
-	math.randomseed(os.time())
+	math.randomseed(posix.getpid())
 	local rand = math.floor(math.random() * (math.ceil(range / port_interval)))
 	local port_base = port_base_start + (rand * port_interval)
 
-	print(("base-port.rand-end is %f"):format(range / port_interval))
-	print(("base-port.rand is %f"):format(rand))
-	print(("base-port.range is %d"):format(range + port_interval))
-	print(("base-port.offset is %d"):format(rand * port_interval))
-	print(("base-port is %d"):format(port_base))
+	print(("... using tcp-port = %d as start port"):format(port_base))
 
 	return port_base
 end
@@ -115,6 +114,9 @@ ADMIN_PORT	   = os.getenv("ADMIN_PORT")		or tostring(port_base + 15)
 ADMIN_MASTER_PORT  = os.getenv("ADMIN_MASTER_PORT")	or tostring(port_base + 25)
 ADMIN_SLAVE_PORT   = os.getenv("ADMIN_SLAVE_PORT")	or tostring(port_base + 35)
 ADMIN_CHAIN_PORT   = os.getenv("ADMIN_CHAIN_PORT")	or tostring(port_base + 45)
+ADMIN_USER	   = os.getenv("ADMIN_USER")		or "root"
+ADMIN_PASSWORD	   = os.getenv("ADMIN_PASSWORD")	or ""
+ADMIN_DEFAULT_SCRIPT_FILENAME = os.getenv("ADMIN_DEFAULT_SCRIPT_FILENAME")	or ""
 -- local PROXY_TMP_LUASCRIPT = os.getenv("PROXY_TMP_LUASCRIPT") or "/tmp/proxy.tmp.lua"
 
 local srcdir		 = os.getenv("srcdir")		 	or testdir .. "/"
@@ -144,6 +146,9 @@ default_proxy_options = {
 	["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. MYSQL_PORT,
 	["proxy-address"]		   	= PROXY_HOST .. ":" .. PROXY_PORT,
 	["admin-address"]		   	= PROXY_HOST .. ":" .. ADMIN_PORT,
+	["admin-username"]		   	= ADMIN_USER,
+	["admin-password"]		   	= ADMIN_PASSWORD,
+	["admin-lua-script"]			= ADMIN_DEFAULT_SCRIPT_FILENAME,
 	["pid-file"]				= PROXY_PIDFILE,
 	["proxy-lua-script"]		= DEFAULT_SCRIPT_FILENAME,
 	["plugin-dir"]			= PROXY_LIBPATH,
@@ -154,6 +159,9 @@ default_master_options = {
 	["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. MYSQL_PORT,
 	["proxy-address"]		   	= PROXY_HOST .. ":" .. PROXY_MASTER_PORT,
 	["admin-address"]		   	= PROXY_HOST .. ":" .. ADMIN_MASTER_PORT,
+	["admin-username"]		   	= ADMIN_USER,
+	["admin-password"]		   	= ADMIN_PASSWORD,
+	["admin-lua-script"]			= ADMIN_DEFAULT_SCRIPT_FILENAME,
 	["pid-file"]				= PROXY_MASTER_PIDFILE,
 	["proxy-lua-script"]		= DEFAULT_SCRIPT_FILENAME,
 	["plugin-dir"]			= PROXY_LIBPATH,
@@ -164,6 +172,9 @@ default_slave_options = {
 	["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. MYSQL_PORT,
 	["proxy-address"]		   	= PROXY_HOST .. ":" .. PROXY_SLAVE_PORT,
 	["admin-address"]		  	= PROXY_HOST .. ":" .. ADMIN_SLAVE_PORT,
+	["admin-username"]		   	= ADMIN_USER,
+	["admin-password"]		   	= ADMIN_PASSWORD,
+	["admin-lua-script"]			= ADMIN_DEFAULT_SCRIPT_FILENAME,
 	["pid-file"]				= PROXY_SLAVE_PIDFILE,
 	["proxy-lua-script"]		= DEFAULT_SCRIPT_FILENAME,
 	["plugin-dir"]			= PROXY_LIBPATH,
@@ -292,11 +303,32 @@ function os_execute(cmdline)
 	return os.execute(cmdline)
 end
 
+---
+-- get the PID from the pid file
+--
 function get_pid(pid_file_name)
-	local fh = assert(io.open(pid_file_name, 'r'))
-	local pid = assert(fh:read() ,
-		"PID not found in " .. pid_file_name)
-	fh:close()
+	-- the file may exist, but the PID may not be written yet
+	local pid
+	local rounds = 0
+
+	repeat 
+		local fh = assert(io.open(pid_file_name, 'r'))
+		pid = fh:read("*n")
+		fh:close()
+		if not pid then
+			glib2.usleep(200 * 1000) -- wait a bit until we get some content
+			rounds = rounds + 1
+
+			if rounds > 10 then
+				error(("reading PID from existing pid-file '%s' failed after waiting 2 sec"):format(
+					pid_file_name))
+			end
+		end
+	until pid
+
+	-- the PID we get here should be a number
+	assert(type(pid) == "number")
+
 	return pid
 end
 
@@ -513,6 +545,7 @@ function run_test(filename, basedir)
 			database = MYSQL_DB,
 			host	 = PROXY_HOST,
 			port	 = PROXY_PORT,
+			verbose  = (VERBOSE > 0) and "TRUE" or "FALSE", -- pass down the verbose setting 
 			["test-file"] = basedir .. "/t/" .. testname .. ".test",
 			["result-file"] = basedir .. "/r/" .. testname .. ".result",
 			["logdir"] = builddir, -- the .result dir might not be writable
@@ -731,10 +764,14 @@ function chain_proxy (backend_lua_scripts, second_lua_script, use_replication)
 			["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. MYSQL_PORT,
 			["proxy-address"]		   = backend_addresses[i],
 			["admin-address"]		   = PROXY_HOST .. ":" .. (ADMIN_CHAIN_PORT + i - 1),
+			["admin-username"]		   	= ADMIN_USER,
+			["admin-password"]		   	= ADMIN_PASSWORD,
+			["admin-lua-script"]		= ADMIN_DEFAULT_SCRIPT_FILENAME,
 			["pid-file"]				= PROXY_CHAIN_PIDFILE .. i,
 			["proxy-lua-script"]		= backend_lua_script or DEFAULT_SCRIPT_FILENAME,
 			["plugin-dir"]			= PROXY_LIBPATH,
 			["basedir"]					= PROXY_TEST_BASEDIR,
+			["log-level"]			= (VERBOSE == 4) and "debug" or "critical",
 		}
 		-- 
 		-- if replication was not started, then it is started here
@@ -752,10 +789,14 @@ function chain_proxy (backend_lua_scripts, second_lua_script, use_replication)
 			["proxy-backend-addresses"] = backend_addresses ,
 			["proxy-address"]		   	= PROXY_HOST .. ":" .. PROXY_PORT,
 			["admin-address"]		   	= PROXY_HOST .. ":" .. ADMIN_PORT,
+			["admin-username"]		   	= ADMIN_USER,
+			["admin-password"]		   	= ADMIN_PASSWORD,
+			["admin-lua-script"]		= ADMIN_DEFAULT_SCRIPT_FILENAME,
 			["pid-file"]				= PROXY_PIDFILE,
 			["proxy-lua-script"]		= second_lua_script or DEFAULT_SCRIPT_FILENAME,
 			["plugin-dir"]			= PROXY_LIBPATH,
 			["basedir"]					= PROXY_TEST_BASEDIR,
+			["log-level"]			= (VERBOSE == 3) and "debug" or "critical",
 	}
 	start_proxy('second_proxy',second_proxy_options) 
 end
@@ -783,30 +824,32 @@ if #arg then
 		-- if it is a directory, execute all of them
 		if stat.mode == "directory" then
 			for file in lfs.dir(a .. "/t/") do
-				local testname = file:match("(.+\.test)$")
-		
-				if testname then
-					print_verbose("# >> " .. testname .. " started")
-		
-					num_tests = num_tests + 1
-					local r, skipped  = run_test("t/" .. testname, a)
-					if (r == 0) then
-						num_passes = num_passes + 1 - skipped
-					else
-						num_fails = num_fails + 1
-						all_ok = false
-						table.insert(failed_test, testname)
-					end
-					num_skipped = num_skipped + skipped
+				if not TESTS_REGEX or file:match(TESTS_REGEX) then
+					local testname = file:match("(.+\.test)$")
+			
+					if testname then
+						print_verbose("# >> " .. testname .. " started")
+			
+						num_tests = num_tests + 1
+						local r, skipped  = run_test("t/" .. testname, a)
+						if (r == 0) then
+							num_passes = num_passes + 1 - skipped
+						else
+							num_fails = num_fails + 1
+							all_ok = false
+							table.insert(failed_test, testname)
+						end
+						num_skipped = num_skipped + skipped
 
-					print_verbose("# << (exitcode = " .. r .. ")" )
-		
-					if r ~= 0 and exitcode == 0 then
-						exitcode = r
+						print_verbose("# << (exitcode = " .. r .. ")" )
+			
+						if r ~= 0 and exitcode == 0 then
+							exitcode = r
+						end
+					end
+					if all_ok == false and (not FORCE_ON_ERROR) then
+						break
 					end
-				end
-				if all_ok == false and (not FORCE_ON_ERROR) then
-					break
 				end
 			end
 		else 
