@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2008, 2009, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
 #include <glib.h>
@@ -49,6 +51,7 @@ void t_network_backend_new() {
 void t_network_backends_add() {
 	network_backends_t *backends;
 
+	g_log_set_always_fatal(G_LOG_FATAL_MASK);
 	backends = network_backends_new();
 	g_assert(backends);
 	
@@ -69,6 +72,9 @@ void t_network_backends_add() {
 	
 	g_assert_cmpint(network_backends_count(backends), ==, 1);
 
+	/* make sure bad port numbers also fail */
+	g_assert_cmpint(network_backends_add(backends, "127.0.0.1:113306", BACKEND_TYPE_RW), ==, -1);
+	
 	network_backends_free(backends);
 }
 
