@@ -1,18 +1,20 @@
 /* $%BEGINLICENSE%$
- Copyright (C) 2008 MySQL AB, 2008 Sun Microsystems, Inc
+ Copyright (c) 2008, 2009, Oracle and/or its affiliates. All rights reserved.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License as
+ published by the Free Software Foundation; version 2 of the
+ License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ 02110-1301  USA
 
  $%ENDLICENSE%$ */
 
@@ -20,14 +22,35 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#ifndef _WIN32
+#include <signal.h>
+#endif
+
+#ifndef WIN32
+#include <unistd.h>
+#include <stdlib.h>
+#include <fcntl.h>
+#endif /* WIN32 */
 
 #include <glib.h>
+#include <glib/gstdio.h> /* for g_unlink */
 
 #include "network-socket.h"
 
 #if GLIB_CHECK_VERSION(2, 16, 0)
 #define C(x) x, sizeof(x) - 1
 
+#ifndef WIN32
+#define LOCAL_SOCK "/tmp/mysql-proxy-test.socket"
+
+typedef struct {
+	char	sockname[sizeof(LOCAL_SOCK) + 10];
+} local_unix_t;
+
+static local_unix_t	*pp = NULL;
+static local_unix_t local_test_arg;
+#endif /* WIN32 */
+
 void test_network_socket_new() {
 	network_socket *sock;
 
@@ -292,6 +315,115 @@ void t_network_socket_connect_udp(void) {
 	network_socket_free(server);
 }
 
+#ifndef WIN32
+
+/**
+ * test if _is_local() works on unix-domain sockets
+ *
+ * MacOS X 10.4 doesn't report a .sa_family for one of the sides of the connection if it is a unix-socket 
+ */
+void t_network_socket_is_local_unix() {
+	network_socket *s_sock; /* the server side socket, listening for requests */
+	network_socket *c_sock; /* the client side socket, that connects */
+	network_socket *a_sock; /* the server side, accepted socket */
+
+	g_test_bug("42220");
+	g_log_set_always_fatal(G_LOG_FATAL_MASK); /* gtest modifies the fatal-mask */
+
+	s_sock = network_socket_new();
+	network_address_set_address(s_sock->dst, "/tmp/mysql-proxy-test.socket");
+
+	c_sock = network_socket_new();
+	network_address_set_address(c_sock->dst, "/tmp/mysql-proxy-test.socket");
+
+
+	/* hack together a network_socket_accept() which we don't have in this tree yet */
+	g_assert_cmpint(NETWORK_SOCKET_SUCCESS, ==, network_socket_bind(s_sock));
+
+	g_assert_cmpint(NETWORK_SOCKET_SUCCESS, ==, network_socket_connect(c_sock));
+
+	a_sock = network_socket_accept(s_sock);
+	g_assert(a_sock);
+
+	g_assert_cmpint(TRUE, ==, network_address_is_local(s_sock->dst, a_sock->dst));
+
+	network_socket_free(a_sock);
+	network_socket_free(c_sock);
+	network_socket_free(s_sock);
+
+	g_unlink("/tmp/mysql-proxy-test.socket");
+}
+
+void t_network_localsocket_setup(local_unix_t *p, gconstpointer G_GNUC_UNUSED _test_data) {
+	g_assert(p != NULL);
+	snprintf(p->sockname, sizeof(p->sockname), LOCAL_SOCK ".%d", (int)getpid());
+	pp = p;
+}
+
+void t_network_localsocket_teardown(local_unix_t *p, gconstpointer G_GNUC_UNUSED _test_data) {
+	g_assert(p != NULL);
+	if (p->sockname[0] != '\0') {
+		(void) g_unlink(p->sockname);
+		p->sockname[0] = '\0';
+	}
+	pp = NULL;
+}
+
+void exitfunc(int sig) {
+	if (pp != NULL && pp->sockname[0] != '\0')
+		(void) g_unlink(pp->sockname);
+
+	abort();
+}
+
+/**
+ * test if local sockets are removed at shutdown
+ * this is an extension of _is_local_unix(), therefore looks much like it;
+ * just in case, we leave all tests from that function in as well.
+ */
+void t_network_socket_rem_local_unix(local_unix_t *p, gconstpointer G_GNUC_UNUSED _test_data) {
+	network_socket *s_sock; /* the server side socket, listening for requests */
+	network_socket *c_sock; /* the client side socket, that connects */
+	network_socket *a_sock; /* the server side, accepted socket */
+
+	/*
+	 * if an assertion fails, we receive the ABORT signal. We need to
+	 * close the unix socket in that case too. The regular teardown function
+	 * is not called in this case, so we install our own handler.
+	 */
+	signal(SIGABRT, exitfunc);
+	g_test_bug("42220");
+	g_log_set_always_fatal(G_LOG_FATAL_MASK); /* gtest modifies the fatal-mask */
+
+	s_sock = network_socket_new();
+	network_address_set_address(s_sock->dst, p->sockname);
+
+	c_sock = network_socket_new();
+	network_address_set_address(c_sock->dst, p->sockname);
+
+	g_assert_cmpint(NETWORK_SOCKET_SUCCESS, ==, network_socket_bind(s_sock));
+
+	g_assert_cmpint(g_access(p->sockname, 0), ==, 0);
+
+	g_assert_cmpint(NETWORK_SOCKET_SUCCESS, ==, network_socket_connect(c_sock));
+
+	a_sock = network_socket_accept(s_sock);
+	g_assert(a_sock);
+
+	g_assert_cmpint(TRUE, ==, network_address_is_local(s_sock->dst, a_sock->dst));
+
+	network_socket_free(a_sock);
+	g_assert_cmpint(g_access(p->sockname, 0), ==, 0);
+	network_socket_free(c_sock);
+	g_assert_cmpint(g_access(p->sockname, 0), ==, 0);
+	network_socket_free(s_sock);
+	g_assert_cmpint(g_access(p->sockname, 0), ==, -1);
+
+	/* re-establish default signal disposition */
+	signal(SIGABRT, SIG_DFL);
+}
+
+#endif /* WIN32 */
 
 int main(int argc, char **argv) {
 	g_test_init(&argc, &argv, NULL);
@@ -303,6 +435,14 @@ int main(int argc, char **argv) {
 	g_test_add_func("/core/network_queue_append", test_network_queue_append);
 	g_test_add_func("/core/network_queue_peek_string", test_network_queue_peek_string);
 	g_test_add_func("/core/network_queue_pop_string", test_network_queue_pop_string);
+#ifndef WIN32
+	g_test_add_func("/core/network_socket_is_local_unix",t_network_socket_is_local_unix);
+
+	g_test_add("/core/network_socket_rem_local_unix", local_unix_t,
+			&local_test_arg,
+			t_network_localsocket_setup, t_network_socket_rem_local_unix, 
+			t_network_localsocket_teardown);
+#endif /* WIN32 */
 #if 0
 	/**
 	 * disabled for now until we fixed the _to_read() on HP/UX and AIX (and MacOS X)
@@ -314,8 +454,8 @@ int main(int argc, char **argv) {
 
 	return g_test_run();
 }
-#else
+#else /* GLIB_CHECK_VERSION */
 int main() {
 	return 77;
 }
-#endif
+#endif /* GLIB_CHECK_VERSION */
